@@ -5,6 +5,7 @@ import time
 
 import feedparser
 from dateutil import parser
+from datetime import timedelta
 
 from .ProjectHolder import ProjectHolder
 from .utils import asset_does_not_belong_to_machine, ApiCredentialsError, BadProjectError
@@ -14,6 +15,13 @@ class GitHubRepoSession(ProjectHolder):
     DEFAULT_HOSTNAME = 'github.com'
     DEFAULT_HOLDER = True
 
+    # one-word aliases or simply known popular repos to skip using search API
+    KNOWN_REPOS_BY_NAME = {
+        'php': {'repo': 'php/php-src'},
+        'linux': {'repo': 'torvalds/linux'},
+        'kernel': {'repo': 'torvalds/linux'},
+    }
+
     def __init__(self, repo, hostname):
         super(GitHubRepoSession, self).__init__()
         self.api_token = os.getenv("GITHUB_API_TOKEN")
@@ -79,11 +87,10 @@ class GitHubRepoSession(ProjectHolder):
                             time.sleep(wait_for)
                         self.rate_limited_count = self.rate_limited_count + 1
                         return self.get(url)
-                    else:
-                        raise ApiCredentialsError(
-                            'Exceeded API rate limit after waiting: {}'.format(
-                                r.json()['message'])
-                        )
+                    raise ApiCredentialsError(
+                        'Exceeded API rate limit after waiting: {}'.format(
+                            r.json()['message'])
+                    )
             return self.get(url)
 
         if r.status_code == 403 and url != self.get_rate_limit_url():
@@ -116,6 +123,144 @@ class GitHubRepoSession(ProjectHolder):
             return r.json()
         return None
 
+    # faster tag search: aggregates highest semver between records of 100 (faster search in --major)
+    # much fewer requests
+    def find_in_tags_via_graphql(self, ret, pre_ok, major):
+        query_fmt = """
+        {
+          repository(owner: "%s", name: "%s") {
+            refs(refPrefix: "refs/tags/", first: 100, after: "%s", orderBy: {field:
+            TAG_COMMIT_DATE,
+            direction: DESC}) {
+              edges {
+                node {
+                  name
+                  target {
+                    oid
+                    ... on Tag {
+                      message
+                      commitUrl
+                      tagger {
+                        name
+                        email
+                        date
+                      }
+                    }
+                  }
+                }
+                cursor
+              }
+            }
+          }
+        }
+        """
+        cursor = ''
+
+        while not ret:
+            # testing on php/php-src
+            owner, name = self.repo.split('/')
+            query = query_fmt % (owner, name, cursor)
+            r = self.post('{}/graphql'.format(self.api_base), json={'query': query})
+            if r.status_code != 200:
+                return ret
+            j = r.json()
+            # no tags
+            if not j['data']['repository']['refs']['edges']:
+                break
+            for edge in j['data']['repository']['refs']['edges']:
+                node = edge['node']
+                cursor = edge['cursor']
+                tag_name = node['name']
+                version = self.sanitize_version(tag_name, pre_ok, major)
+                if not version:
+                    continue
+                d = node['target']['tagger']['date']
+                tag_date = parser.parse(d)
+
+                if not ret or version > ret['version'] or tag_date > ret['tag_date']:
+                    # we always want to return formal release if it exists, cause it has useful data
+                    # grab formal release via APi to check for pre-release mark
+                    r = self.repo_query('/releases/tags/{}'.format(tag_name))
+                    if r.status_code == 200:
+                        formal_release = r.json()
+                        if not pre_ok and formal_release['prerelease']:
+                            log.info(
+                                "Found formal release for this tag which is unwanted "
+                                "pre-release: {}.".format(version))
+                            continue
+                        if ret and tag_date + timedelta(days=365) < ret['tag_date']:
+                            log.info('The version {} is newer, but is too old!'.format(version))
+                            break
+                        # use full release info
+                        ret = formal_release
+                        ret['tag_name'] = tag_name
+                        ret['tag_date'] = tag_date
+                        ret['version'] = version
+                        ret['type'] = 'graphql'
+                    else:
+                        if ret and tag_date + timedelta(days=365) < ret['tag_date']:
+                            log.info('The version {} is newer, but is too old!'.format(version))
+                            break
+                        ret = {
+                            'tag_name': tag_name,
+                            'tag_date': tag_date,
+                            'version': version,
+                            'type': 'graphql'
+                        }
+        return ret
+
+    # finding in tags requires paging through ALL of them, because the API does not list them
+    # in order of recency, thus this is very slow
+    # in: current release to be returned, output: newer release to be returned
+    def find_in_tags(self, ret, pre_ok, major):
+        r = self.repo_query('/tags?per_page=100')
+        if r.status_code != 200:
+            return None
+        tags = r.json()
+        while 'next' in r.links.keys():
+            r = self.get(r.links['next']['url'])
+            tags.extend(r.json())
+
+        for t in tags:
+            tag_name = t['name']
+            version = self.sanitize_version(tag_name, pre_ok, major)
+            if not version:
+                continue
+            c = self.repo_query('/git/commits/{}'.format(t['commit']['sha']))
+            d = c.json()['committer']['date']
+            d = parser.parse(d)
+
+            if not ret or version > ret['version'] or d > ret['tag_date']:
+                # rare case: if upstream filed formal pre-release that passes as stable
+                # version (tag is 1.2.3 instead of 1.2.3b) double check if pre-release
+                # TODO handle API failure here as it may result in "false positive"?
+                release_for_tag = None
+                if not pre_ok:
+                    r = self.repo_query('/releases/tags/{}'.format(tag_name))
+                    if r.status_code == 200:
+                        # noinspection SpellCheckingInspection
+                        if r.json()['prerelease']:
+                            log.info(
+                                "Found formal release for this tag which is unwanted "
+                                "pre-release: {}.".format(version))
+                            continue
+                        release_for_tag = r.json()
+
+                log.info("Setting version as current selection: {}.".format(version))
+                if release_for_tag:
+                    ret = release_for_tag
+                    ret['tag_name'] = tag_name
+                    ret['tag_date'] = parser.parse(release_for_tag['published_at'])
+                    ret['version'] = version
+                    ret['type'] = 'release'
+                else:
+                    ret = t
+                    ret['tag_name'] = tag_name
+                    ret['tag_date'] = d
+                    ret['version'] = version
+                    ret['type'] = 'tag'
+        return ret
+
     def get_latest(self, pre_ok=False, major=None):
         """
         Gets latest release satisfying "prereleases are OK" or major/branch constraints
@@ -149,13 +294,11 @@ class GitHubRepoSession(ProjectHolder):
         # API requests are varied by cookie, we don't want serializer for cache fail because of that
         self.cookies.clear()
         feed = feedparser.parse(r.text)
-        # TODO choose topmost (most recent), but do a doble check whether the edit was released (
-        #  tag api)
         for tag in feed.entries:
             # https://github.com/apache/incubator-pagespeed-ngx/releases/tag/v1.13.35.2-stable
             tag_name = tag['link'].split('/')[-1]
             version = self.sanitize_version(tag_name, pre_ok, major)
-            if version:
+            if version and (not ret or ret['version'] < version):
                 # we always want to return formal release if it exists, cause it has useful data
                 # grab formal release via APi to check for pre-release mark
                 r = self.repo_query('/releases/tags/{}'.format(tag_name))
@@ -166,16 +309,24 @@ class GitHubRepoSession(ProjectHolder):
                             "Found formal release for this tag which is unwanted "
                             "pre-release: {}.".format(version))
                         continue
+                    tag_date = parser.parse(tag['updated'])
+                    if ret and tag_date + timedelta(days=365) < ret['tag_date']:
+                        log.info('The version {} is newer, but is too old!'.format(version))
+                        break
                     # use full release info
                     ret = formal_release
                     ret['tag_name'] = tag_name
-                    ret['tag_date'] = parser.parse(tag['updated'])
+                    ret['tag_date'] = tag_date
                     ret['version'] = version
                     ret['type'] = 'feed'
                 else:
+                    tag_date = parser.parse(tag['updated'])
+                    if ret and tag_date + timedelta(days=365) < ret['tag_date']:
+                        log.info('The version {} is newer, but is too old!'.format(version))
+                        break
                     ret = tag
                     ret['tag_name'] = tag_name
-                    ret['tag_date'] = parser.parse(tag['updated'])
+                    ret['tag_date'] = tag_date
                     ret['version'] = version
                     ret['type'] = 'feed'
                     # remove keys which are non-jsonable
@@ -183,7 +334,6 @@ class GitHubRepoSession(ProjectHolder):
                     ret.pop('updated_parsed', None)
                     ret.pop('published_parsed', None)
                 log.info("Selected version as current selection: {}.".format(version))
-                break
         # we are good with release from feeds only without looking at the API
         # simply because feeds list stuff in order of recency
         if ret:
@@ -229,47 +379,11 @@ class GitHubRepoSession(ProjectHolder):
 
         # formal release may not exist at all, or be "late/old" in case
         # actual release is only a simple tag so let's try /tags
-        r = self.repo_query('/tags?per_page=100')
-        if r.status_code == 200:
-            for t in r.json():
-                tag_name = t['name']
-                version = self.sanitize_version(tag_name, pre_ok, major)
-                if not version:
-                    continue
-                c = self.repo_query('/git/commits/{}'.format(t['commit']['sha']))
-                d = c.json()['committer']['date']
-                d = parser.parse(d)
-
-                if not ret or version > ret['version'] or d > ret['tag_date']:
-                    # rare case: if upstream filed formal pre-release that passes as stable
-                    # version (tag is 1.2.3 instead of 1.2.3b) double check if pre-release
-                    # TODO handle API failure here as it may result in "false positive"?
-                    release_for_tag = None
-                    if not pre_ok:
-                        r = self.repo_query('/releases/tags/{}'.format(tag_name))
-                        if r.status_code == 200:
-                            # noinspection SpellCheckingInspection
-                            if r.json()['prerelease']:
-                                log.info(
-                                    "Found formal release for this tag which is unwanted "
-                                    "pre-release: {}.".format(version))
-                                continue
-                            else:
-                                release_for_tag = r.json()
-
-                    log.info("Setting version as current selection: {}.".format(version))
-                    if release_for_tag:
-                        ret = release_for_tag
-                        ret['tag_name'] = tag_name
-                        ret['tag_date'] = parser.parse(release_for_tag['published_at'])
-                        ret['version'] = version
-                        ret['type'] = 'release'
-                    else:
-                        ret = t
-                        ret['tag_name'] = tag_name
-                        ret['tag_date'] = d
-                        ret['version'] = version
-                        ret['type'] = 'tag'
+        if "GITHUB_API_TOKEN" in os.environ:
+            # GraphQL requires auth
+            ret = self.find_in_tags_via_graphql(ret, pre_ok, major)
+        else:
+            ret = self.find_in_tags(ret, pre_ok, major)
 
         return ret
 
